import pandas as pd
import numpy as np
import json

from emod_api.demographics.Demographics import Demographics, Node

class TBHIVDemographics(Demographics):
    """
    This class is derived from emod_api.demographics' Demographics class so that we can set 
    certain defaults for TBHIV in construction. Keen observers will note thatt SetDefaultProperties
    does not look like a TBHIV-specific function. But as we add other disease types the 
    generatlizations and speicfics will become clearer. The architectural point is solid.
    """
    def __init__(self, pop, nodes=None, idref="Gridded world grump2.5arcmin", base_file=None):
        if nodes is None:
            nodes = [ Node( pop=pop, lat=0, lon=0, forced_id=1 ) ]
        super().__init__( nodes )
        super().SetDefaultNodeAttributes()
        self.SetHIVCoInfectionDistribution()
        self.SetHIVTBCoInfectionMortalityDistribution()
        self.SetHeteroRiskUniformDist( 1.0, 1.0 ) # Uniform from 1 to 1 is a way of saying 1. emod-api doesnot

    def SetHIVCoInfectionDistribution( self ):
        """
        Insert some notion of a default HIVCoInfection distribution.
        """
        default_hivco = { "HIVCoinfectionDistribution":
            {
                "NumDistributionAxes": 3,
                "AxisNames": [ "gender", "time", "age" ],
                "AxisUnits": [ "male =0, female=1", "years", "years" ],
                "AxisScaleFactors": [ 1, 365, 365 ],
                "NumPopulationGroups": [ 2, 3, 4 ],
                "PopulationGroups":
                [
                    [ 0, 1 ],
                    [ 0, 50, 100],
                    [ 0, 15, 35, 50]
                ],
                "ResultUnits": "Probability of HIV infection per repetition of HIV Outbreak",
                "ResultScaleFactor": 0.019199999999999998,
                "ResultValues":
                [

                    [
                        [1, 1, 1, 1],
                        [1, 1, 1, 1],
                        [1, 1, 1, 1]

                    ],

                     [
                        [1, 1, 1, 1],
                        [1, 1, 1, 1],
                        [1, 1, 1, 1]

                    ]


                ]
            }
        }
        self.raw['Defaults']['IndividualAttributes'].update( 
            default_hivco 
        )

    def SetHIVTBCoInfectionMortalityDistribution( self ):
        """
        Insert some notion of a default HIVTBCoInfection Mortality distribution.
        """
        default_setting = {
            "HIVTBCoinfMortalityDistribution": {
                "AxisNames": [
                    "age",
                    "year"
                ],
                "AxisScaleFactors": [
                    365,
                    1
                ],
                "AxisUnits": [
                    "years",
                    "years"
                ],
                "NumDistributionAxes": 2,
                "NumPopulationGroups": [
                    2,
                    1
                ],
                "PopulationGroups": [
                    [
                        0,
                        15
                    ],
                    [
                        0
                    ]
                ],
                "ResultScaleFactor": 2.739726027397e-06,
                "ResultUnits": "annual deaths per 1000 individuals",
                "ResultValues": [
                    [
                        14
                    ],
                    [
                        14
                    ]
                ]
            }
        }
        self.raw['Defaults']['IndividualAttributes'].update( 
            default_setting 
        )


#
# 'STATIC' instantiator functions that are really module functions
#

def fromBasicNode(lat=0, lon=0, pop=1e6, name=1, forced_id=1, implicit_config_fns = None ):
    """
    This function creates a single-node TBHIVDemographics instance from the params you give it. 
    """
    new_nodes = [ Node(lat=lat, lon=lon, pop=pop, name=name, forced_id=forced_id) ]
    return TBHIVDemographics(nodes=new_nodes, pop=pop)

def fromData( pop=1e6, filename_male='Malawi_male_mortality.csv', filename_female='Malawi_female_mortality.csv' ):
    # User variables where to start extrapolation (ie before HIV epidemic) and how far to look back
    # All in years unless noted otherwise
    extrap_point = 1980
    look_back = 20
    time_horizon = 2100
    now_ish = 1980
    look_back_from_now_ish = 20

    interval_for_fitting_regression = [1970, 1980]
    predict_horizons = 2050

    # This is all you need to use here (Note you prob want to override the default on extrapolation
    myob = TBHIVDemographics( pop )
    #myobj.mortality( filename_male, filename_female ) #creatrs object
    myob.infer_natural_mortality( filename_male, filename_female, interval_for_fitting_regression)
    #myob.generate_file( name='Output_demog.json' )
    return myob



